from flask import Flask, jsonify
from flask_socketio import SocketIO
from flask_cors import CORS
from helpers.Database import Database

# helpers
from helpers.ds1820 import Ds1820
from helpers.Lcd import Lcd
from helpers.UltrasonicSensor import UltrasonicSensor
from helpers.Button import Button
from helpers.Mcp import Mcp
from helpers.PWM_led import PWM_led


from RPi import GPIO
import json
import threading
import time


GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)

# ip
from subprocess import check_output
ip = ""

# neopixel ledstrip
import board
import neopixel
ORDER = neopixel.GRB
num_pixels = 28
pixels = neopixel.NeoPixel(board.D18, num_pixels, brightness=1, auto_write=False, pixel_order=ORDER)
pixels.fill((0, 0, 0))
pixels.show()

# One wire sensor
sensor = Ds1820()

# lcd
lcd = Lcd()
LCD_LINE_1 = 0x80
LCD_LINE_2 = 0xC0

# ultrasone sensor
ultrasonicSensor = UltrasonicSensor(23, 24)

# servo
container1 = 5
GPIO.setup(container1, GPIO.OUT)
container1_pwm = GPIO.PWM(container1, 50)

container2 = 6
GPIO.setup(container2, GPIO.OUT)
container2_pwm = GPIO.PWM(container2, 50)

container3 = 13
GPIO.setup(container3, GPIO.OUT)
container3_pwm = GPIO.PWM(container3, 50)

container4 = 26
GPIO.setup(container4, GPIO.OUT)
container4_pwm = GPIO.PWM(container4, 50)

# force sensor
forceSensor = 0
mcp = Mcp(1, 2)

# button
button = Button(27)

# button rgb led
pwm_red = PWM_led(25)
pwm_green = PWM_led(22)
pwm_blue = PWM_led(17)

pwm_red.change_duty_cycle(100)
pwm_green.change_duty_cycle(100)
pwm_blue.change_duty_cycle(100)

#buzzer
Buzzer = 12
GPIO.setup(Buzzer, GPIO.OUT)


# variables
connected = False
scheduleToday = ""
alarm = False
stop = False
stopGreen = False
isButtonClicked = False
oneMinOver = False
twoMinOver = False
scheduleStatus = ""
endTimeOnMin  = time.time()
endTimeTwoMin = time.time()
scheduleBusy = False



app = Flask(__name__)
CORS(app)

socketio = SocketIO(app)
endpoint = '/api/v1'
conn = Database(app, 'mct', 'mct', 'medicationdb', '169.254.10.1')


@socketio.on("connect")
def connecting():
    global connected
    socketio.emit("connected")
    if not connected:
        lcd.lcd_init()
        get_ips()
        show_next_medication()
        connected = True
    print("Connection with client established")


def get_ips():
    global ip, scheduleStatus, oneMinOver, twoMinOver, scheduleStatus, endTimeOnMin, endTimeTwoMin, isButtonClicked
    scheduleStatus = ""
    oneMinOver = False
    twoMinOver = False
    scheduleStatus = ""
    endTimeOnMin = time.time()
    endTimeTwoMin = time.time()
    isButtonClicked = False

    pwm_red.change_duty_cycle(100)
    pwm_green.change_duty_cycle(100)
    pwm_blue.change_duty_cycle(100)

    pixels.fill((0,0,0))
    pixels.show()

    ips = check_output(['hostname', '--all-ip-addresses']).decode(encoding='utf-8')
    pos = ips.find(' ')
    if pos != -1:
        ip = ips[pos + 1:]
        ip = ip[:ip.find(' ')]

    lcd.lcd_string(ip, LCD_LINE_1)
    lcd.lcd_string("", LCD_LINE_2)


def show_next_medication():
    global scheduleToday, scheduleBusy
    schedules = getNextScheduleToday()
    print(schedules)
    if schedules == "[]":
        lcd.lcd_string("No schedule", LCD_LINE_2)
    else:
        schedules = json.loads(schedules)
        for schedule in schedules:
            currentTime = check_output(['date', '+%H:%M']).decode(encoding='utf-8').rstrip("\n")

            timeArray = schedule["TimeDue"].split(":");

            if len(timeArray[0]) == 2:
                timeOfSchedule = (timeArray[0]) + ":" + timeArray[1]
            else:
                timeOfSchedule = ("0" + timeArray[0]) + ":" + timeArray[1]

            if timeOfSchedule >= currentTime:
                lcd.lcd_string("Next Pill: {0}".format(timeOfSchedule), LCD_LINE_2)
                scheduleToday = schedule

                scheduleBusy = False
                checkSchedule()
                break

            else:
                lcd.lcd_string("No schedule", LCD_LINE_2)


def checkSchedule():
    global  alarm, endTimeOnMin, stop, stopGreen, scheduleBusy
    currentTime = check_output(['date', '+%H:%M']).decode(encoding='utf-8').rstrip("\n")
    timeArray = scheduleToday["TimeDue"].split(":");

    if len(timeArray[0]) == 2:
        timeOfSchedule = (timeArray[0]) + ":" + timeArray[1]
    else:
        timeOfSchedule = ("0" + timeArray[0]) + ":" + timeArray[1]

    print(currentTime)
    print(timeOfSchedule)
    if currentTime == timeOfSchedule and alarm == False:
        scheduleBusy = True
        alarm = True
        stop = False
        stopGreen = False
        lcd.lcd_string("Time for your", LCD_LINE_1)
        lcd.lcd_string("medication:{}".format(timeOfSchedule), LCD_LINE_2)
        endTimeOnMin = time.time() + 10
        startOneMin()

        blink_led_green()
        playAlarmSound()
        startLedstrip_Green()

        distance = round(ultrasonicSensor.distance(), 2)
        print(distance)
        addSensorDataToDb("Ultrasonic sensor", distance, "Check how far away the patient is")

        print("alarm")


    if not scheduleBusy:
        threading.Timer(60, checkSchedule).start()


def startOneMin():
    global  oneMinOver, endTimeOnMin, endTimeTwoMin, stopGreen
    print("one-min " + str(oneMinOver))
    print(time.time())
    print(endTimeOnMin)
    if time.time() <= endTimeOnMin and isButtonClicked is False:
        threading.Timer(1, startOneMin).start()
    else:
        if isButtonClicked is False:
            stopGreen = True
            oneMinOver = True
            blink_led_orange()
            startLedstrip_Orange()
            endTimeTwoMin = time.time() + 10
            startTwoMin()


def startTwoMin():
    print("two-min: " + str(twoMinOver))
    if time.time() <= endTimeTwoMin and isButtonClicked is False:
        threading.Timer(1, startTwoMin).start()
    else:
        notTaken()

def notTaken():
    global scheduleStatus, twoMinOver, stop, stopGreen, alarm, scheduleStatus, endTimeTwoMin
    if scheduleStatus == "" and isButtonClicked == False:
        stop = True
        stopGreen = True
        twoMinOver = True

        time.sleep(1)
        lcd.lcd_string("you forgot ", LCD_LINE_1)
        lcd.lcd_string("your medication", LCD_LINE_2)

        for i in range(num_pixels):
            pixels[i] = (204, 0, 0)
            pixels.show()
            time.sleep(50 / 1000.0)

        for i in range(num_pixels):
            pixels[i] = (204, 0, 0)
            pixels.show()
            time.sleep(50 / 1000.0)

        time.sleep(4)

        scheduleStatus = "Not taken"
        print("Not taken")
        AddPatientMedicationHistory(scheduleStatus)
        get_ips()
        show_next_medication()
        alarm = False



def read_button(pin):
    global knop, stop, alarm, stopGreen, isButtonClicked, scheduleStatus, oneMinOver, twoMinOver
    print("The button was pressed")
    if alarm == True:
        stop = True
        stopGreen = True
        alarm = False

        if oneMinOver is False and twoMinOver is False:
            scheduleStatus = "Taken"
            isButtonClicked = True
            print("taken")

            pixels.fill((0, 200, 13))
            pixels.show()

            pwm_red.change_duty_cycle(100)
            pwm_green.change_duty_cycle(0)
            pwm_blue.change_duty_cycle(95)

        elif oneMinOver is True and twoMinOver is False:
            scheduleStatus = "Late"
            print("late")
            isButtonClicked = True

            pixels.fill((230, 92, 0))
            pixels.show()

            pwm_red.change_duty_cycle(0)
            pwm_green.change_duty_cycle(48)
            pwm_blue.change_duty_cycle(100)


        temperature = round(sensor.temperature, 2)
        if temperature != 0:
            addSensorDataToDb("One wire", temperature, "check the temperature in the dispenser")

        checkCupPresent()


button.on_press(read_button)


def checkCupPresent():
    if scheduleStatus == "Taken":
        pixels.fill((0, 200, 13))
        pixels.show()

        pwm_red.change_duty_cycle(100)
        pwm_green.change_duty_cycle(0)
        pwm_blue.change_duty_cycle(95)
    elif scheduleStatus == "Late":
        pixels.fill((230, 92, 0))
        pixels.show()

        pwm_red.change_duty_cycle(0)
        pwm_green.change_duty_cycle(48)
        pwm_blue.change_duty_cycle(100)

    forceSensor_data = mcp.read_channel(forceSensor)
    addSensorDataToDb("Force sensor", forceSensor_data, "Check if a cup is present underneath dispenser")
    print(forceSensor_data)

    if  forceSensor_data > 5:
        cupPresent = True
        lcd.lcd_string("Please wait ...", LCD_LINE_1)
        lcd.lcd_string("", LCD_LINE_2)

    else:
        cupPresent = False
        lcd.lcd_string("Please place a", LCD_LINE_1)
        lcd.lcd_string("cup".format(time), LCD_LINE_2)

    if not cupPresent:
        threading.Timer(3, checkCupPresent).start()
    else:
        getmedicationFromDispenser()




def getmedicationFromDispenser():
    currentMedications = getScheduleTodayTime(scheduleToday["TimeDue"])
    currentMedications = json.loads(currentMedications)

    for medication in currentMedications:
        container = medication["container"]
        for number in range(0, medication["Frequancy"]):
            if container == "Container 1":
                rotate_servo(container1_pwm)
            elif container == "Container 2":
                rotate_servo(container2_pwm)
            elif container == "Container 3":
                rotate_servo(container3_pwm)
            elif container == "Container 4":
                rotate_servo(container4_pwm)

    lcd.lcd_string("medication", LCD_LINE_1)
    lcd.lcd_string("is ready", LCD_LINE_2)

    checkCupGone()


def checkCupGone():
    global oneMinOver, twoMinOver, stop, stopGreen
    forceSensor_data = mcp.read_channel(forceSensor)
    addSensorDataToDb("Force sensor", forceSensor_data, "Check if a cup is present underneath dispenser")

    if forceSensor_data > 5:
        threading.Timer(3, checkCupGone).start()
    else:
        AddPatientMedicationHistory(scheduleStatus)
        get_ips()
        show_next_medication()
        oneMinOver = False
        twoMinOver = False
        stop = False
        stopGreen = False



def AddPatientMedicationHistory(value):
    patient = conn.get_data(
        'SELECT P.PatientId, P.FirstName, P.LastName, P.DateOfBirth, P.Information, G.Name as gender FROM patient as P Join gender as G on P.GenderId = G.GenderId;')

    conn.set_data(
        "INSERT INTO patientmedicationhistory(IsTaken, PatientId, PatientMedicationInfoId) VALUES(%s, %s, %s)",
        (value, patient[0]["PatientId"], scheduleToday["patientMedicationinfoId"]))


@socketio.on("getPatientInformation")
def getPatientInformation(patientId):
    patientInformation = conn.get_data("Select * from patientmedicationhistory where PatientId = %s order by Datetime Asc", patientId)
    result = json.dumps(patientInformation, default=set_default)
    socketio.emit("patientInformation", result)

@socketio.on("checkPatientExists")
def doesPatientExists():
    patient_data = conn.get_data('SELECT * FROM patient;')
    if patient_data == ():
        exists = False
    else:
        exists = True
        patient = conn.get_data('SELECT P.PatientId, P.FirstName, P.LastName, P.DateOfBirth, P.Information, G.Name as gender FROM patient as P Join gender as G on P.GenderId = G.GenderId;')
        result = json.dumps(patient, default=set_default)
        socketio.emit("Patient", result)

    socketio.emit("patientExists", exists)


def set_default(obj):
    return str(obj)


def addPatient(data):
    patient_data = conn.get_data('SELECT * FROM patient;')
    if patient_data == ():
        genderId = conn.get_data("Select GenderId from gender where Name=%s", data["Gender"])
        conn.set_data("INSERT INTO patient(FirstName, LastName, DateOfBirth, GenderId, Information) VALUES(%s, %s, %s, %s, %s)",
                      (data['FirstName'], data['LastName'], data['DateOfBirth'], genderId[0]["GenderId"], data['Info']))

    socketio.emit("AddedPatient")


@socketio.on("UpdatePatient")
def updatePatient(data):

    genderId = conn.get_data("Select GenderId from gender where Name=%s", data["Gender"])
    print(genderId[0]["GenderId"])
    conn.set_data('update patient set FirstName = %s, LastName = %s, DateOfBirth = %s,GenderId = %s, Information = %s where PatientId = %s',[data['FirstName'], data['LastName'], data['DateOfBirth'], genderId[0]["GenderId"], data['Info'], data['PatientId']])

    socketio.emit("AddedPatient")

@socketio.on("removePatient")
def removePatient(data):
    conn.set_data('DELETE FROM patientmedication WHERE PatientId=%s', data)
    conn.set_data('DELETE FROM patientmedicationhistory WHERE PatientId=%s', data)
    conn.set_data('DELETE FROM patient WHERE PatientId=%s', data)

@socketio.on("getContainers")
def getContainers():
    containers = conn.get_data('SELECT * FROM  container WHERE  containerId not in (SELECT ContainerId FROM medication WHERE ContainerId IS NOT NULL) Group by Name;')
    socketio.emit("containers", containers)\


@socketio.on("addMedication")
def addMedication(data):
    if data['Container'] != "none":
        containerId = conn.get_data("Select ContainerId from container where Name=%s", data["Container"])
        conn.set_data(
            "INSERT INTO medication(Name, Description, ContainerId) VALUES(%s, %s, %s)",
            (data['Name'], data['Description'],  containerId[0]["ContainerId"]))
    else:
        conn.set_data(
            "INSERT INTO medication(Name, Description) VALUES(%s, %s)",
            (data['Name'], data['Description']))


@socketio.on("getMedications")
def getMedications():
    medications = conn.get_data('SELECT m.MedicationId, m.Name, m.Description, c.Name as container FROM  medication as m Left join container as c on m.containerId = c.containerId Order By m.Name ASC;')
    socketio.emit("medications", medications)

@socketio.on("getMedicationsInContainer")
def getMedicationsInContainer():
    medications = conn.get_data('SELECT m.MedicationId, m.Name, m.Description, c.Name as container FROM  medication as m join container as c on m.containerId = c.containerId Order By m.Name ASC;')
    socketio.emit("medicationsInContainer", medications)


@socketio.on("updateMedication")
def updateMedication(data):
    if data["Container"] == "none":
        conn.set_data('update medication set Name = %s, Description = %s, ContainerId = %s where MedicationId = %s', [data['Name'], data['Description'], None, data["MedicationId"]])
    else:
        containerId = conn.get_data("Select ContainerId from container where Name=%s", data["Container"])
        conn.set_data('update medication set Name = %s, Description = %s, ContainerId = %s where MedicationId = %s', [data['Name'], data['Description'], containerId[0]["ContainerId"], data["MedicationId"]])


@socketio.on("removeMedication")
def removeMedication(data):
    if data["Container"] == "null":
        medication = conn.get_data('select m.MedicationId, m.Name, m.Description, c.Name as container from  medication as m Left join container as c on m.containerId = c.containerId where m.name = %s and c.Name is null', data["Name"])
    else:
        medication = conn.get_data('select m.MedicationId, m.Name, m.Description, c.Name as container from  medication as m Left join container as c on m.containerId = c.containerId where m.name = %s and c.Name = %s', (data["Name"], data["Container"]))

    conn.set_data('DELETE FROM patientmedication WHERE MedicationId=%s', medication[0]["MedicationId"])
    conn.set_data('DELETE FROM medication WHERE MedicationId=%s', medication[0]["MedicationId"])

    socketio.emit("removedMedication")
    show_next_medication()


@socketio.on("getMedication")
def getMedication(data):

    if data["ctnr"] == "null":
        medication = conn.get_data('select m.MedicationId, m.Name, m.Description, c.Name as container from  medication as m Left join container as c on m.containerId = c.containerId where m.name = %s and c.Name is null', data["name"])
    else:
        medication = conn.get_data('select m.MedicationId, m.Name, m.Description, c.Name as container from  medication as m Left join container as c on m.containerId = c.containerId where m.name = %s and c.Name = %s', (data["name"], data["ctnr"]))

    socketio.emit("medication", medication)


weekdays = ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday']
@socketio.on("addDoseSchedule")
def addDoseSchedule(data):

    for medication in data['Medications']:
        med = conn.get_data('select m.MedicationId, m.Name, m.Description, c.Name as container from  medication as m Left join container as c on m.containerId = c.containerId where m.name = %s', (medication["medication"]))
        patient = conn.get_data('SELECT P.PatientId, P.FirstName, P.LastName, P.DateOfBirth, P.Information, G.Name as gender FROM patient as P Join gender as G on P.GenderId = G.GenderId;')

        if data["DaysOfTheWeek"][0] == "everyday":
            timeArray = []
            for item in weekdays:
                timeId = conn.set_data(
                    "INSERT INTO time(TimeDue, DayDue) VALUES(%s, %s)",
                    (medication['time'], item))
                timeArray.append(timeId)

            patientmedicationinfoId = conn.set_data(
                "INSERT INTO patientmedicationinfo(StartDate, StopDate, Frequancy, Period) VALUES(%s, %s, %s, %s)",
                (data["StartDate"], data["EndDate"], medication["quantity"], "Everyday of the week"))

            conn.set_data(
                "INSERT INTO patientmedication(PatientId, MedicationId, PatientMedicationInfoId) VALUES(%s, %s, %s)",
                (patient[0]["PatientId"], med[0]["MedicationId"], patientmedicationinfoId))

            for time in timeArray:
                conn.set_data(
                    "INSERT INTO patientmedicationinfotime(TimeId, PatientMedicationInfoId) VALUES(%s, %s)",
                    (time, patientmedicationinfoId))

        else:
            timeArray = []
            for day in data["DaysOfTheWeek"]:
                timeId = conn.set_data(
                    "INSERT INTO time(TimeDue, DayDue) VALUES(%s, %s)",
                    (medication['time'], day))
                timeArray.append(timeId)


            numberOfDays = len(data["DaysOfTheWeek"])
            if numberOfDays == 1:
                period = "1 day"
            else :
                period = str(numberOfDays) + " days"
            patientmedicationinfoId = conn.set_data(
                "INSERT INTO patientmedicationinfo(StartDate, StopDate, Frequancy, Period) VALUES(%s, %s, %s, %s)",
                (data["StartDate"], data["EndDate"], medication["quantity"], period))

            conn.set_data(
                "INSERT INTO patientmedication(PatientId, MedicationId, PatientMedicationInfoId) VALUES(%s, %s, %s)",
                (patient[0]["PatientId"], med[0]["MedicationId"], patientmedicationinfoId))

            for time in timeArray:
                conn.set_data(
                    "INSERT INTO patientmedicationinfotime(TimeId, PatientMedicationInfoId) VALUES(%s, %s)",
                    (time, patientmedicationinfoId))

    show_next_medication()


@socketio.on("getScheduleToday")
def getScheduleToday(front = True):
    schedule = conn.get_data("Select PMI.patientMedicationinfoId, M.Name, PMI.StartDate, PMI.StopDate, PMI.Frequancy, T.TimeDue, T.DayDue, C.Name as container from patientmedicationinfo as PMI JOIN patientmedication as PM on PM.patientMedicationinfoId = PMI.patientMedicationinfoId Join medication as M on M.medicationId = PM.medicationId Join container as C on M.ContainerId = C.containerID JOIN patientmedicationinfotime as PMIT on PMIT.patientMedicationinfoId = PM.patientMedicationinfoId Join time as T on T.timeId = PMIT.timeId WHERE CURDATE() BETWEEN PMI.StartDate and PMI.StopDate and DayDue = DAYNAME(CURDATE()) Order by TimeDue Asc;")
    result = json.dumps(schedule, default=set_default)
    if front == True:
        socketio.emit("schedule", result)
    else:
        return result


def getNextScheduleToday():
    schedule = conn.get_data("Select PMI.patientMedicationinfoId, M.Name, PMI.StartDate, PMI.StopDate, PMI.Frequancy, T.TimeDue, T.DayDue, C.Name as container from patientmedicationinfo as PMI JOIN patientmedication as PM on PM.patientMedicationinfoId = PMI.patientMedicationinfoId Join medication as M on M.medicationId = PM.medicationId Join container as C on M.ContainerId = C.containerID JOIN patientmedicationinfotime as PMIT on PMIT.patientMedicationinfoId = PM.patientMedicationinfoId Join time as T on T.timeId = PMIT.timeId WHERE CURDATE() BETWEEN PMI.StartDate and PMI.StopDate and DayDue = DAYNAME(CURDATE()) and PMI.patientMedicationinfoId not in (Select patientMedicationinfoId from patientmedicationhistory) Order by TimeDue Asc;")
    result = json.dumps(schedule, default=set_default)
    return result


def getScheduleTodayTime(time):
    schedule = conn.get_data("Select M.Name, PMI.StartDate, PMI.StopDate, PMI.Frequancy, T.TimeDue, T.DayDue, C.Name as container from patientmedicationinfo as PMI JOIN patientmedication as PM on PM.patientMedicationinfoId = PMI.patientMedicationinfoId Join medication as M on M.medicationId = PM.medicationId Join container as C on M.ContainerId = C.containerID JOIN patientmedicationinfotime as PMIT on PMIT.patientMedicationinfoId = PM.patientMedicationinfoId Join time as T on T.timeId = PMIT.timeId WHERE CURDATE() BETWEEN PMI.StartDate and PMI.StopDate and DayDue = DAYNAME(CURDATE()) and TimeDue = %s Order by TimeDue Asc;", time)
    result = json.dumps(schedule, default=set_default)
    return result

@socketio.on("getScheduleDay")
def getScheduleDay(data):
    schedule = conn.get_data("Select M.Name, PMI.StartDate, PMI.StopDate, PMI.Frequancy, T.TimeDue, T.DayDue from patientmedicationinfo as PMI JOIN patientmedication as PM on PM.patientMedicationinfoId = PMI.patientMedicationinfoId Join medication as M on M.medicationId = PM.medicationId JOIN patientmedicationinfotime as PMIT on PMIT.patientMedicationinfoId = PM.patientMedicationinfoId Join time as T on T.timeId = PMIT.timeId WHERE %s BETWEEN PMI.StartDate and PMI.StopDate and DayDue = DAYNAME(%s) Order by TimeDue Asc;", (data, data))
    result = json.dumps(schedule, default=set_default)
    socketio.emit("schedule", result)



@socketio.on("checkMedicationTaken")
def checkMedicationTaken(date):
    medicationStatus = conn.get_data("SELECT PMH.DateTime, PMH.IsTaken, M.Name, T.TimeDue FROM patientmedicationhistory PMH Join patientmedication as PM on PMH.PatientMedicationInfoId = PM.PatientMedicationInfoId Join medication as M on M.medicationId = PM.medicationId  JOIN patientmedicationinfotime as PMIT on PMIT.patientMedicationinfoId = PM.patientMedicationinfoId  Join time as T on T.timeId = PMIT.timeId where DATE(DateTime) = %s and IsTaken=%s and DayDue = DAYNAME(%s);", (date, "Taken", date))
    result = json.dumps(medicationStatus, default=set_default)
    socketio.emit("medicationStatusTaken", result)


@socketio.on("checkMedicationNotTaken")
def checkMedicationNotTaken(date):
    medicationStatus = conn.get_data("SELECT PMH.DateTime, PMH.IsTaken, M.Name, T.TimeDue FROM patientmedicationhistory PMH Join patientmedication as PM on PMH.PatientMedicationInfoId = PM.PatientMedicationInfoId Join medication as M on M.medicationId = PM.medicationId  JOIN patientmedicationinfotime as PMIT on PMIT.patientMedicationinfoId = PM.patientMedicationinfoId  Join time as T on T.timeId = PMIT.timeId where DATE(DateTime) = %s and IsTaken=%s and DayDue = DAYNAME(%s);", (date, "Not taken", date))
    result = json.dumps(medicationStatus, default=set_default)
    socketio.emit("medicationStatusNotTaken", result)


@socketio.on("checkMedicationtTakenLate")
def checkMedicationtTakenLate(date):
    medicationStatus = conn.get_data("SELECT PMH.DateTime, PMH.IsTaken, M.Name, T.TimeDue FROM patientmedicationhistory PMH Join patientmedication as PM on PMH.PatientMedicationInfoId = PM.PatientMedicationInfoId Join medication as M on M.medicationId = PM.medicationId  JOIN patientmedicationinfotime as PMIT on PMIT.patientMedicationinfoId = PM.patientMedicationinfoId  Join time as T on T.timeId = PMIT.timeId where DATE(DateTime) = %s and IsTaken=%s and DayDue = DAYNAME(%s);", (date, "Late", date))
    result = json.dumps(medicationStatus, default=set_default)
    socketio.emit("medicationStatusNotTaken", result)


@socketio.on("getTemperature")
def showSensorDataTemp():
    sensorId = conn.get_data("Select SensorId from sensor where Name = %s", "One wire")
    sensortDate = conn.get_data("Select * from sensorhistory where SensorId = %s Limit 25", sensorId[0]["SensorId"])
    result = json.dumps(sensortDate, default=set_default)
    socketio.emit("showSensorDataTemp", result)


@socketio.on("getTakenDays")
def getStatusDay(month):
    StatusDays = conn.get_data("Select distinct Date(DateTime), IsTaken from patientmedicationhistory where MONTH(Datetime) = %s and IsTaken = %s", (month, 'Taken'))
    result = json.dumps(StatusDays, default=set_default)
    socketio.emit("takenDays", result)

@socketio.on("getLateDays")
def getStatusDay(month):
    StatusDays = conn.get_data("Select distinct Date(DateTime), IsTaken from patientmedicationhistory where MONTH(Datetime) = %s and IsTaken = %s", (month, 'Late'))
    result = json.dumps(StatusDays, default=set_default)
    socketio.emit("lateDays", result)

@socketio.on("getNotTakenDays")
def getStatusDay(month):
    StatusDays = conn.get_data("Select distinct Date(DateTime), IsTaken from patientmedicationhistory where MONTH(Datetime) = %s and IsTaken = %s", (month, 'Not Taken'))
    result = json.dumps(StatusDays, default=set_default)
    socketio.emit("notTakenDays", result)


def addSensorDataToDb(sensor, value, action ):
    sensorId = conn.get_data("Select SensorId from sensor where Name = %s", sensor)
    conn.set_data("INSERT INTO sensorhistory(Value, SensorId, action) VALUES(%s, %s, %s)",(value, sensorId[0]["SensorId"], action))

def playAlarmSound():
    GPIO.output(Buzzer,GPIO.HIGH)
    time.sleep(.1)
    GPIO.output(Buzzer,GPIO.LOW)
    time.sleep(.1)
    GPIO.output(Buzzer,GPIO.HIGH)
    time.sleep(.1)
    GPIO.output(Buzzer,GPIO.LOW)
    time.sleep(.1)
    GPIO.output(Buzzer,GPIO.HIGH)
    time.sleep(.1)

    GPIO.output(Buzzer,GPIO.LOW)
    time.sleep(.2)
    GPIO.output(Buzzer,GPIO.HIGH)
    time.sleep(.2)
    GPIO.output(Buzzer,GPIO.LOW)
    time.sleep(.2)
    GPIO.output(Buzzer,GPIO.HIGH)
    time.sleep(.2)
    GPIO.output(Buzzer,GPIO.LOW)
    time.sleep(.2)
    GPIO.output(Buzzer,GPIO.HIGH)
    time.sleep(.2)
    GPIO.output(Buzzer,GPIO.LOW)
    time.sleep(.2)

    GPIO.output(Buzzer,GPIO.HIGH)
    time.sleep(.1)
    GPIO.output(Buzzer,GPIO.LOW)
    time.sleep(.1)
    GPIO.output(Buzzer,GPIO.HIGH)
    time.sleep(.1)
    GPIO.output(Buzzer,GPIO.LOW)
    time.sleep(.1)
    GPIO.output(Buzzer,GPIO.HIGH)
    time.sleep(.1)
    GPIO.output(Buzzer,GPIO.LOW)
    time.sleep(.7)

    if not stop:
        threading.Timer(1, playAlarmSound).start()
    else:
        GPIO.output(Buzzer, GPIO.LOW)

def blink_led_green():
    pwm_red.change_duty_cycle(100)
    pwm_green.change_duty_cycle(0)
    pwm_blue.change_duty_cycle(95)
    time.sleep(.4)
    pwm_red.change_duty_cycle(100)
    pwm_green.change_duty_cycle(100)
    pwm_blue.change_duty_cycle(100)
    time.sleep(.4)

    if not stopGreen:
        threading.Timer(.4, blink_led_green).start()


def blink_led_orange():
    pwm_red.change_duty_cycle(0)
    pwm_green.change_duty_cycle(48)
    pwm_blue.change_duty_cycle(100)
    time.sleep(.4)
    pwm_red.change_duty_cycle(100)
    pwm_green.change_duty_cycle(100)
    pwm_blue.change_duty_cycle(100)
    time.sleep(.4)

    if not stop:
        threading.Timer(.4, blink_led_orange).start()


def rotate_servo(pwm_container):
    if pwm_container == container1_pwm:
        pwm_container.start(0)
        pwm_container.ChangeDutyCycle(8.5)
        time.sleep(0.55)
        pwm_container.ChangeDutyCycle(0)
        time.sleep(1.5)
    elif pwm_container == container2_pwm or pwm_container == container3_pwm:
        pwm_container.start(0)
        pwm_container.ChangeDutyCycle(8.5)
        time.sleep(0.7)
        pwm_container.ChangeDutyCycle(0)
        time.sleep(1.5)
    elif pwm_container == container4_pwm:
        pwm_container.start(0)
        pwm_container.ChangeDutyCycle(8.5)
        time.sleep(0.75)
        pwm_container.ChangeDutyCycle(0)
        time.sleep(1.5)


def startLedstrip_Green():

    for i in range(num_pixels):
        pixels[i] = (0, 200, 13)
        pixels.show()
        time.sleep(50 / 1000.0)

    for i in range(num_pixels):
        pixels[i] = (0, 0, 0)
        pixels.show()
        time.sleep(50 / 1000.0)

    if not stopGreen:
        threading.Timer(1, startLedstrip_Green).start()


def startLedstrip_Orange():

    for i in range(num_pixels):
        pixels[i] = (255, 102, 0)
        pixels.show()
        time.sleep(50 / 1000.0)

    for i in range(num_pixels):
        pixels[i] = (0, 0, 0)
        pixels.show()
        time.sleep(50 / 1000.0)

    if not stop:
        threading.Timer(1, startLedstrip_Orange).start()

if __name__ == '__main__':
    socketio.run(app, host='0.0.0.0', port=5000)



